Package: rJava
Version: 1.0-18
Title: Low-Level R to Java Interface
Author: Simon Urbanek [aut, cre, cph] (https://urbanek.nz, ORCID:
    <https://orcid.org/0000-0003-2297-1732>)
Maintainer: Simon Urbanek <Simon.Urbanek@r-project.org>
Authors@R: person("Simon", "Urbanek", role=c("aut","cre","cph"), email="Simon.Urbanek@r-project.org", comment=c("https://urbanek.nz", ORCID="0000-0003-2297-1732"))
Depends: R (>= 3.6.0), methods
Description: Low-level interface to Java VM very much like .C/.Call and friends. Allows creation of objects, calling methods and accessing fields.
License: LGPL-2.1
URL: https://www.rforge.net/rJava/
SystemRequirements: Java JDK 1.2 or higher (for JRI/REngine JDK 1.4 or
        higher), GNU make
BugReports: https://github.com/s-u/rJava/issues
NeedsCompilation: yes
Packaged: 2026-04-08 02:23:57 UTC; rforge
Repository: CRAN
Date/Publication: 2026-04-08 12:21:52 UTC
